/*************************************************************
	ControlModule.c Ver.2.1

						2015.7.17 M.Ustumi@ARCDEVICE

**************************************************************/

#include"stdio.h"
#include"math.h"
#include"timer.h"
#include"ControlModule.h"

volatile BYTE CM_bID = 0;
const BYTE CM_bVersion = 0x21; // version 2.1

BYTE CM_bStatus = CM_NORMAL;
BYTE CM_bCtrlStatus = CM_NORMAL;

volatile WORD CM_wCtrlTimeoutCount = 0;	//  ^CAEgJEg
volatile WORD CM_wCtrlTimeout = 1000;	//  ^CAEg
volatile WORD CM_wComTimeout = 1000;	//  ʐM^CAEg

volatile BOOL CM_SyncCommandReady = FALSE;
volatile BOOL CM_AsyncCommandReady = FALSE;

WORD wRxBufferA[ BUF_SIZE ];
WORD wRxBufferB[ BUF_SIZE ];
WORD wTxBufferA[ BUF_SIZE ];
WORD wTxBufferB[ BUF_SIZE ];
BYTE* pbRxBuffer;
BYTE* pbTxBuffer;
WORD wTxBufCountA=0;
WORD wTxBufCountB=0;
WORD* pwTxBufCount;
BOOL RxBufAEnable;
BOOL TxBufAEnable;
BYTE bChecksum;

WORD _EEDATA(2) EEP_wID=1;	//vOݎ̏ ID

BYTE ReadID( void )
{
	WORD w;

	EEP_Read( 
		__builtin_tblpage(&EEP_wID),
		__builtin_tbloffset(&EEP_wID),
	 	&w, 1 );
	return( (BYTE)w );
}

void WriteID( BYTE bID )
{
	WORD w;

	w = bID;
	EEP_Write( 
		__builtin_tblpage(&EEP_wID),
		__builtin_tbloffset(&EEP_wID),
	 	&w, 1 );
}

void CM_Open( void )
{
	BYTE b;

	b = CM_OPEN;
	CM_WriteBuf( &b, 1 );
	CM_Reply();
}

void CM_Close( void )
{
	BYTE b;

	b = CM_CLOSE;
	CM_WriteBuf( &b, 1 );
	CM_Reply();
}

void CM_Reset( void )
{
	CM_Reply();
	asm( "RESET" );
}

void CM_Ping( void )
{
	BYTE b;

	b = CM_PING;
	CM_WriteBuf( &b, 1 );

	CM_Reply();
}

void CM_SetID( void )
{
	BYTE b;
	
	CM_ReadBuf( &b, 1);
	WriteID( b );
	CM_Reply();
	CM_bID = b;
}

void CM_GetID( void )
{
	CM_bID = ReadID();
	CM_WriteBuf( (BYTE*)&CM_bID, 1 );
	CM_Reply();
}

void CM_GetVer( void )
{
	CM_WriteBuf( (BYTE*)&CM_bVersion, 1 );
	CM_Reply();
}

void CM_Blink( void )
{
	BYTE b;

	CM_ReadBuf( &b, 1 );
	CM_Reply();

	UT_StatusBlink( 125, b, 4 );
}

void CM_SetParam( void )
{
	CM_PARAM param;

	CM_ReadBuf( (BYTE*)&param, sizeof(CM_PARAM) );
	CM_Reply();

	UT_StopTimer();
			
	CM_wCtrlTimeout = (WORD)(param.fCtrlTimeout*1000.0f);
	CM_wComTimeout = (WORD)(param.fComTimeout*1000.0f);
	ESPI1_SetTimeout( CM_wComTimeout );

	UT_SetTimer( param.dt * 1000.0f );
}

void CM_GetParam( void )
{
	CM_PARAM param; 

	param.dt = UT_dt_ms / 1000.0f;
	param.fCtrlTimeout = (float)CM_wCtrlTimeout/1000.0f;
	param.fComTimeout  = (float)CM_wComTimeout /1000.0f;
	CM_WriteBuf( (BYTE*)&param, sizeof(CM_PARAM) );

	CM_Reply();
}

void CM_Init( void )
{
	CM_bStatus = CM_NORMAL;

	CM_bID = ReadID();	// ID

	UT_ZeroMemory( wRxBufferA, BUF_SIZE  );
	UT_ZeroMemory( wRxBufferB, BUF_SIZE  );
	UT_ZeroMemory( wTxBufferA, BUF_SIZE  );
	UT_ZeroMemory( wTxBufferB, BUF_SIZE  );

	RxBufAEnable = FALSE;
	pbRxBuffer = (BYTE*)wRxBufferA;
	TxBufAEnable = FALSE;
	pbTxBuffer = (BYTE*)wTxBufferA;
	wTxBufCountA = 0;
	wTxBufCountB = 0;
	pwTxBufCount = &wTxBufCountA;
	bChecksum = 0;

	ESPI1_Open();
	ESPI1_IDReceiveEnable();

	IPC2bits.SPI1IP = 5; // priority
	IFS0bits.SPI1IF = 0;
	IEC0bits.SPI1IE = 1;

	UT_SetTimer(1.0f);
}

void CM_ChangeRxBuffer( void )
{
	if( RxBufAEnable ){
		RxBufAEnable = FALSE;
		pbRxBuffer = (BYTE*)wRxBufferA;
	}else{
		RxBufAEnable = TRUE;
		pbRxBuffer = (BYTE*)wRxBufferB;
	}
}

void CM_ChangeTxBuffer( void )
{
	if( TxBufAEnable ){
		TxBufAEnable = FALSE;
		pbTxBuffer = (BYTE*)wTxBufferA;
		wTxBufCountA = 0;
		pwTxBufCount = &wTxBufCountA;
	}else{
		TxBufAEnable = TRUE;
		pbTxBuffer = (BYTE*)wTxBufferB;
		wTxBufCountB = 0;
		pwTxBufCount = &wTxBufCountB;
	}
}

void CM_ReadBuf( BYTE* pbBuf, WORD wSize )
{
	while( wSize-->0){
		*pbBuf++ = *pbRxBuffer++;
	}
}

void CM_WriteBuf( BYTE* pbBuf, WORD wSize )
{
	while( wSize-->0){
		*pbTxBuffer++ = *pbBuf;
		bChecksum += *pbBuf++;
		(*pwTxBufCount)++;
	}
}

BOOL CM_CheckSum( BYTE* pbBuffer, WORD wSize )
{
	BYTE bSum=0;

	wSize--;
	while( wSize-- ){
		bSum += *pbBuffer++;	
	}
	if( bSum != *pbBuffer )	return( FALSE );

	return( TRUE );
}

void CM_AddChecksum( void )
{
	*pbTxBuffer++ = bChecksum;
	(*pwTxBufCount)++;
	bChecksum = 0;

}

BOOL CM_SyncTransceive( void )
{
	WORD wSize, wWordSize, w;
	WORD *pwBuf;
	BYTE *ptr;

// ʐM\ł邱Ƃ`

	ESPI1_WriteEnable();
	w = ACK;
	if( !ESPI1_Write( &w, 1 ) ) return( FALSE );
	ESPI1_WriteDisable();

// ǂݍ݂ɐ؂ւ
	if( !ESPI1_ReadEnable() ) return( FALSE );
	
// Mf[^TCYǂݍ	
	if( !ESPI1_Read( &wSize, 1 ) ) return( FALSE );

// wߒlȂǂ̃f[^Sbufɗߍ	
// Mobt@̐؂ւ	
	if( RxBufAEnable ) pwBuf = wRxBufferA;
	else pwBuf = wRxBufferB;

// Rx Data 
	wWordSize = wSize+1;
	wWordSize = wWordSize >> 1;
	if( !ESPI1_Read( pwBuf, wWordSize ) ) return( FALSE );
	ESPI1_ReadDisable();

// `FbNT
	if( !CM_CheckSum( (BYTE*)pwBuf, wSize ) ){
		CM_bStatus = CM_COMMAND_SUM_ERROR;
		
		wSize = 2; // status + checksum
		ptr = (BYTE*)&w;
		ptr[0] = CM_bStatus;
		ptr[1] = CM_bStatus;

		ESPI1_WriteEnable();
		if( !ESPI1_Write( &wSize, 1 ) ) return( FALSE );
		if( !ESPI1_Write( &w, 1 ) ) return( FALSE );
		ESPI1_WriteDisable();

		return( FALSE );
	}
	return( TRUE );
}

BOOL CM_AsyncTransceive( void )
{
	WORD wSize, wWordSize, w;
	WORD *pwBuf;
	BYTE *ptr;

	if( TxBufAEnable ){
		pwBuf = wTxBufferA;
		wSize = wTxBufCountA;
	}else{
		pwBuf = wTxBufferB;
		wSize = wTxBufCountB;
	}

	ESPI1_WriteEnable();
	if( !ESPI1_Write( &wSize, 1 ) ){
		CM_bStatus = CM_REPLY_ERROR + ESPI1_bStatus;
		return(FALSE);
	}

	wWordSize = wSize + 1;
	wWordSize = wWordSize >> 1;

	if( !ESPI1_Write( pwBuf, wWordSize ) ){
		CM_bStatus = CM_REPLY_ERROR + ESPI1_bStatus;
		return(FALSE);
	}
	ESPI1_WriteDisable();
	
// ǂݍ݂ɐ؂ւ
	if(!ESPI1_ReadEnable() ) return( FALSE );

// Mf[^TCYǂݍ	
	if( !ESPI1_Read( &wSize, 1 ) ) return( FALSE );

// wߒlȂǂ̃f[^Sbufɗߍ	
// Mobt@̐؂ւ	
	if( RxBufAEnable ) pwBuf = wRxBufferA;
	else pwBuf = wRxBufferB;

	wWordSize = wSize + 1;
	wWordSize = wWordSize >> 1;
// Rx Data 
	if( !ESPI1_Read( pwBuf, wWordSize ) ) return( FALSE );
	ESPI1_ReadDisable();
	CM_Rearm();	// M̏	

// `FbNT
	if( !CM_CheckSum( (BYTE*)pwBuf, wSize ) ){
		CM_bStatus = CM_COMMAND_SUM_ERROR;
		return( FALSE );
	}	
	return( TRUE );
}

BOOL CM_Reply( void )
{
	WORD wSize, wWordSize;
	WORD *pwBuf;
	BYTE *ptr;

// Mobt@̃Zbg

	CM_WriteBuf( &CM_bStatus, 1 );
	CM_AddChecksum();
	CM_ChangeTxBuffer();
	
	if( TxBufAEnable ){
		pwBuf = wTxBufferA;
		wSize = wTxBufCountA;
	}else{
		pwBuf = wTxBufferB;
		wSize = wTxBufCountB;
	}
	
	ESPI1_WriteEnable();
	if( !ESPI1_Write( &wSize, 1 ) ){
		CM_bStatus = CM_REPLY_ERROR + ESPI1_bStatus;
		return(FALSE);
	}

	wWordSize = wSize + 1;
	wWordSize = wWordSize >> 1;

	if( !ESPI1_Write( pwBuf, wWordSize ) ){
		CM_bStatus = CM_REPLY_ERROR + ESPI1_bStatus;
		return(FALSE);
	}
	ESPI1_WriteDisable();

	return( TRUE );	
}

void CM_PrepareReply( void )
{
	CM_WriteBuf( &CM_bStatus, 1 );
	CM_AddChecksum();
	CM_ChangeTxBuffer();
}


void CM_Rearm( void )
{
	if( SPI1CONbits.SSEN == 0 ){
		ESPI1_IDReceiveEnable();

		IFS0bits.SPI1IF = 0;
		IEC0bits.SPI1IE = 1;
	}
}

void __attribute__((__interrupt__, no_auto_psv)) _SPI1Interrupt( void )
{
	WORD w;
	BYTE *pb;
	BYTE* pBuf;

	//ID̃`FbN
	ESPI1_Read( &w, 1 );
	IFS0bits.SPI1IF = 0;
	
	pb = (BYTE*)&w;
	
	if( (pb[0] != CM_bID)&&(pb[0] != CM_MASTER_ID) ) return;

	ESPI1_IDReceiveDisable();
	IEC0bits.SPI1IE = 0;

	if( pb[1] < 100 ){
		if( !CM_SyncTransceive() ){
			CM_bStatus = CM_COMMAND_ERROR + ESPI1_bStatus;
			CM_Rearm();
			return;
		}
		CM_SyncCommandReady = TRUE;
		CM_AsyncCommandReady = FALSE;

	}else{
		if( !CM_AsyncTransceive() ){
			CM_bStatus = CM_COMMAND_ERROR + ESPI1_bStatus;
			CM_Rearm();
			return;
		}
		CM_SyncCommandReady = FALSE;
		CM_AsyncCommandReady = TRUE;
	}	
}


BOOL CM_Command( void )
{
	WORD w;
	BYTE *pb;

	if( !ESPI1_DataReady() ) return( FALSE );

//ID̃`FbN
	ESPI1_Read( &w, 1 );
	pb = (BYTE*)&w;
	if( (pb[0] != CM_bID)&&(pb[1] != CM_MASTER_ID) ) return( FALSE );
		
	ESPI1_IDReceiveDisable();


	if( !CM_SyncTransceive() ){
		CM_bStatus = CM_COMMAND_ERROR + ESPI1_bStatus;
		CM_Rearm();
		return( FALSE );
	}
	CM_ChangeRxBuffer();

	CM_bStatus = CM_NORMAL;
	return( TRUE );
}



