/****************************************************************
	MotorPWM.h 
	
	Motor control functions for MCD05
	2ch PWM output

							2015.7.14@M.Utsumi@ARCDEVICE

*****************************************************************/

#include"MotorPWM.h"
#include"Encoder.h"

void MOTOR_Open( void )
{
	TRIS_ENABLE = 0;
	ENABLE = 1;	// L6203:ENABLE=ON
 
	PTCON = 0x2000;	// 0010 0000 0000 0000
	/*
		15:   PTEN 	 = 0 PWM time base is off
		14:   N/A
		13:   PTSIDL = 1 PWM time base halts in CPU Idle mode
		12-8: N/A
		7-4: PTOPS	 = 000 1:1 postscale
		3-2: PTCKPS	 = 00 PWM time base input clock period is TCY (1:1 prescale)
		1-0: PTMOD	 = 0 PWM time base operates in Free Running mode
	*/

	PTMR	= 0;	// counter clear
	PTPER = 511;	// Time base period
	SEVTCMP = 0x0000;	
	/*
		15:  SEVTDIR = 0 Special Event Trigger will occur when the PWM time base is counting up
		14-0:SEVTCMP = 0 Special Event Compare Value bits
	*/
	PWMCON1 = 0x0F00; // 0000 1111 0000 0000
	/*
		15-12: N/A
		11:    PMOD4 = 1 PWM I/O pin pair is in Independent Output mode
		10:    PMOD3 = 1 PWM I/O pin pair is in Independent Output mode
		9:     PMOD2 = 1 PWM I/O pin pair is in Independent Output mode
		8:     PMOD1 = 1 PWM I/O pin pair is in Independent Output mode
		7:     PEN4H = 0 PWMxHy pin disabled; I/O pin becomes general purpose I/O
		6:     PEN3H = 0 PWMxHy pin disabled; I/O pin becomes general purpose I/O
		5:     PEN2H = 0 PWMxHy pin disabled; I/O pin becomes general purpose I/O
		4:     PEN1H = 0 PWMxHy pin disabled; I/O pin becomes general purpose I/O
		2:     PEN4L = 0 PWMxHy pin disabled; I/O pin becomes general purpose I/O
		2:     PEN3L = 0 PWMxHy pin disabled; I/O pin becomes general purpose I/O
		1:     PEN2L = 0 PWMxHy pin disabled; I/O pin becomes general purpose I/O
		0:     PEN1L = 0 PWMxHy pin disabled; I/O pin becomes general purpose I/O
	*/
	PWMCON2bits.SEVOPS	= 0; // 1:1 postscale
	PWMCON2bits.IUE		= 0; // Updates to the active PxDC registers are synchronized to the PWM time base
	PWMCON2bits.OSYNC	= 0; // Output overrides via the PxOVDCON register occur on next TCY boundary
	PWMCON2bits.UDIS	= 0; // Updates from du
	PDC1 = 0;	// PWM Duty Cycle

	PTCONbits.PTEN 	= 1;	// PWM time base is on
}

void MOTOR_Close( void )
{
	PTCONbits.PTEN 	= 0;	// PWM time base is off
}


void MOTOR_Drive( int ch, int duty )
{
	unsigned int enb;
	int duty_abs;	 
	
	if( duty > MOTOR_MAX_VALUE ){	// brake
		enb = 1;
		PWMCON1bits.PEN1L = 0;
		PWMCON1bits.PEN1H = 0;
		duty_abs = 0;
		duty = 0;
	}else if( duty == 0 ){	// free
		enb = 0;
		duty_abs = 0;
		duty = 0;
	}else if( duty>0 ){		// CW
		enb = 1;
		PWMCON1bits.PEN1L = 1;
		PWMCON1bits.PEN1H = 0;
		duty_abs =  duty;
	}else{					// CCW
		enb = 1;
		PWMCON1bits.PEN1L = 0;
		PWMCON1bits.PEN1H = 1;
		duty_abs = -duty;
	}

	if( ENCODER_SimMode ) ENCODER_SimCount(duty);
	else{
		PDC1 = duty_abs;
		ENABLE = enb;
	}
}

